% Script for computating the approximate innovation estimators and approximate fitting-filter 
% proposed in Jimenez & Ozaki (2002) for diffusion processes: 
%     dx = f(t,x,theta) dt + g(t,x,theta) dw, 
% given a set of discrete observations of some components of x. 

% These estimators for the uknown parameters theta and unobserved components of the process x are computed following 
% the heuristic introduced in Ozaki et. al (2000). This heuristic essential has three steps: a) computation of the 
% parameters theta; b) computation of the unobserved initial state of x; c) computation of the approxiamte-fitting 
% filter.
% For actual data a carefull selection of the aditional parameters: firstpoint, amplitude as well the options for the 
% optimization algorithm "fmincon" is very important to obtain successful results. In the same way an adequate 
% selection of the initial values for the parameters theta (theta0) and initial state (Xf0, Pf0) are crucial. 
% Thus, import modification must be done in the scripts for the generation and test that run before the present script.  
% Typically, before to run the present script many simulation with the scripts for generation and test must be done 
% to select adecuate values for the variables theta0, X0 and V0. 

% References:
%   -  "An approximate innovation method for the estimation of diffusion processes from discrete data", 
%       Jimenez J.C. & Ozaki, J. Time Series Analysis 
%   -  "Role of the Likelihood Function in the Estimation of Chaos Models", 
%       Ozaki T., Jimenez J.C. & Haggab-Ozaki V., 
%       J. of Time Series Analysis (2000), Vol. 21, 363-387. 


echo off

global deltat deltat_min n_v_e n_m n_obs t0 tTotal X0 Xf0 Pf0 mpts
global Theta FileNames
global gct             % variable to be used in confun.m

%Definicion de todos las constantes
const_OneDim_1;

load data 
Z=datos;

%*********************  Estimation of the unknown model parameters (theta)

LKWindow=[t0 tTotal];
Xf0Index=[];
LB=[0];
UB=[1.5];
ThetaIndex=[2];
Theta0=Theta;
Theta0(2)=0.7;        % convergence ok
%Theta0(2)=0.35;       % convergence ok
%Theta0(2)=0.5125;     % convergence ok

%in the next commands nu comes from filter with parameters Theta (computed in test_onedim.m)
ch='P';
while ~(strcmp(ch,'Y') | strcmp(ch,'N'))
  ch=upper(input('Optimization with a priory statistics on the innovations? [y/n]= ','s'));
end;
if ch=='Y', 
  firstpoint=1;     % starting point for the computation of the innovations statistics
  lastpoint=n_m;    % last point for the computation of the innovations statistics
  nut=nu(:,firstpoint:lastpoint)';
  amplitude=1;     
  nu_stast=amplitude.*[abs(mean(nut)) diag(cov(nut))'];
else  
  nu_stast=[];
end

figure
plot(nu)

%Pf0 should be convenient adjuted if Xf0 are going to be estimated
OptimOptions = optimset('fmincon');
OptimOptions = optimset(OptimOptions,'LargeScale','off','Display','iter','MaxFunEval',1e3);
EstimOptions = estimset(t,Z,Xf0,Pf0,Theta0,FileNames,LKWindow,mpts,nu_stast,LB,UB,OptimOptions);
[Xff,Pff,Xfp,Pfp,nu,LogLK_est,Xf0_Est,Theta_Est]=InnovEstimators(Xf0Index,ThetaIndex,EstimOptions);

disp(['   '])
disp([' Log LK for Theta_Est= ' num2str(-LogLK_est)])
disp(['   '])
disp(['        theta          theta0          theta_est'])
[Theta(ThetaIndex)' Theta0(ThetaIndex)' Theta_Est(ThetaIndex)']

figure;
subplot(1,2,1)
plot(nu);
subplot(1,2,2);
hist(nu,20);
drawnow;

%****************  plot of exact filters vs. approximate-fitting filter (and their variances)
figure
subplot(2,1,1);
plot(t,ms1,t,Xff(1,:));
subplot(2,1,2);
plot(t,Ps1,t,Pff(:,1,1));
drawnow;




