function z=observnl_m(t,x,theta,FileNames,n_obs,seed)
% Generation of an observation vector z of the Space State (SS) model defined 
% by the continuous equation
%        dx = f(t,x,theta) dt + g(t,x,theta) dw, 
% and the dicrete observation equation 
%         Z = h(t,x,theta) + p(t,x,theta)n + e,
% given x.
%
% Imputs:
%   t:      integrations times (1 x n_m)
%   x:      numerical solution obtained by the LL scheme (n_v_e x n_m)
%   theta:  parameters of the differential equation and observation equation (SS Model)
%   FileNames: cell variable with the name of all the functions that define SS Model
%   n_obs:  number of observation equations (dimension of Z)
%
% Output:
%   z:      observation vector (n_obs x n_m)

randn('state',seed);

See_file=FileNames{13};
Lee_file=FileNames{14};
h=FileNames{9};
p=FileNames{11};

[n_v_e,n_m] = size(x);
index=ones(n_obs,1);   
z=[];

if ~strcmp(h,'zero'),
   See=feval(See_file,theta,index);
   z=zeros(n_obs,n_m);
   if norm(See)<eps,
     SeeRc=See;
   else   
      SeeRc = sqrtm(See);
   end   
   for k=1:n_m
     z(:,k) = feval(h,t(k),x(:,k),theta,index) + SeeRc * randn(n_obs,1);
   end
end

if ~strcmp(p,'zero'),
   Lee=feval(Lee_file,theta,index);
   if isempty(z), 
     z=zeros(n_obs,n_m); 
   end
   n=size(feval(p,t(1),x(:,1),theta,index));
   for i=1:n
     if norm(Lee(i))<eps,
       LRc(i)=Lee(i);
     else   
       LRc(i) = sqrtm(Lee(i));
     end;   
   end;
   for k=1:n_m
     temp = feval(p,t(k),x(:,k),theta,index);
     for i=1:n
        z(:,k)= z(:,k) + temp{i} * LRc(i) * randn;
     end;  
   end  
end

